/*
 * Copyright (c) 2000-2001 Sosnoski Software Solutions, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

import java.io.*;
import java.util.*;

import org.xmlpull.v1.*;

/**
 * XMLPull parser wrapper supplying convenience methods for access.
 *
 * @author Dennis M. Sosnoski
 * @version 1.1
 */

public class PullWrapper
{
	/** Parser in use. */
	protected XmlPullParser m_parser;
	
	/**
	 * Constructor. Builds the actual parser.
	 *
	 * @throws XmlPullParserException if error creating parser
	 */
	
	public PullWrapper() throws XmlPullParserException {
		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
		m_parser = factory.newPullParser();
	}
	
	/**
	 * Parse start of element from document. Ignores character data to next
	 * start or end tag, but throws exception if an end tag is seen before a
	 * start tag, or if the start tag seen does not match the expected name.
	 *
	 * @param tag element name expected
	 * @throws IOException if error reading document
	 * @throws XmlPullParserException if expected element not found, or if
	 * other parse error
	 */

	protected void parseStartTag(String tag) 
		throws IOException, XmlPullParserException {
		while (true) {
			switch (m_parser.next()) {
				
				case XmlPullParser.START_TAG:
					if (m_parser.getName().equals(tag)) {
						return;
					}
					// fall through for error handling
				
				case XmlPullParser.END_TAG:
				case XmlPullParser.END_DOCUMENT:
					throw new XmlPullParserException
						("Missing expected start tag " + tag);

			}
		}
	}
	
	/**
	 * Parse end of element from document. Collects character data to the end
	 * tag and returns it with whitespace stripped. Throws an exception if a 
	 * start tag is seen before an end tag, or if the end tag seen does not 
	 * match the expected name.
	 *
	 * @param tag element name expected
	 * @return content text with whitespace stripped
	 * @throws IOException if error reading document
	 * @throws XmlPullParserException if expected element not found, or if
	 * other parse error
	 */

	protected String parseEndTag(String tag) 
		throws IOException, XmlPullParserException {
		String text = null;
		while (true) {
			switch (m_parser.next()) {
				
				case XmlPullParser.TEXT:
					text = m_parser.getText().trim();
					break;
				
				case XmlPullParser.END_TAG:
					if (m_parser.getName().equals(tag)) {
						return text;
					}
					// fall through for error handling
				
				case XmlPullParser.START_TAG:
				case XmlPullParser.END_DOCUMENT:
					throw new XmlPullParserException
						("Missing expected end tag " + tag);

			}
		}
	}
	
	/**
	 * Parse element with text content, returning the content with whitespace
	 * trimmed. Throws an error if the element start tag is not the next tag
	 * seen, or if any child elements are present in the element.
	 *
	 * @param tag element name expected
	 * @return content text with whitespace stripped
	 * @throws IOException if error reading document
	 * @throws XmlPullParserException if expected element not found, or if
	 * other parse error
	 */

	protected String parseElementContent(String tag) 
		throws IOException, XmlPullParserException {
		parseStartTag(tag);
		return parseEndTag(tag);
	}
	
	/**
	 * Get attribute value from current start tag. Throws an error if the
	 * attribute value is not found in the start tag.
	 *
	 * @param name attribute name to be found
	 * @return attribute value text
	 * @throws IOException if error reading document
	 * @throws XmlPullParserException if attribute not present
	 */

	protected String attributeValue(String name) 
		throws IOException, XmlPullParserException {
		String value = m_parser.getAttributeValue(null, name);
		if (value == null) {
			throw new XmlPullParserException("Missing attribute " + name);
		} else {
			return value;
		}
	}
}
